# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**NOTE**: This file was auto-generated using `make README.md`. DO NOT EDIT.

**Version**: AceTime v1.3

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

In v1.3, the `BasicZoneManager` and `ExtendedZoneManager` classes were unified
under a new parent interface `ZoneManager`. This seems to have caused the flash
size to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most
flash size increase proportional to their limited 32kB limit.

Adding the `ZoneManager` interface simplifies a lot of the complexity with
saving and restoring time zones using the `TimeZoneData` object, and I think it
is worth the extra cost of flash size. The mitigating factor is that
applications targetted towards 8-bit processors will normally have fixed number
of timezones at compile time, so they can avoid using a `ZoneManager`, and avoid
this penalty in flash size.

## Arduino Nano

* Arduino IDE 1.8.13
* Arduino AVR Boards 1.8.3

```
+--------------------------------------------------------------+
| Functionality                   |    flash/ram |       Delta |
|---------------------------------+--------------+-------------|
| Baseline                        |    448/   10 |     0/    0 |
|---------------------------------+--------------+-------------|
| LocalDateTime                   |   1648/  123 |  1200/  113 |
| ZonedDateTime                   |   2440/  123 |  1992/  113 |
| Basic TimeZone (1 zone)         |   6910/  187 |  6462/  177 |
| Basic TimeZone (2 zones)        |   7488/  193 |  7040/  183 |
| Basic ZoneManager (1 zone)      |   8374/  213 |  7926/  203 |
| Basic ZoneManager (all)         |  23290/  601 | 22842/  591 |
| Extended TimeZone (1 zone)      |   9690/  187 |  9242/  177 |
| Extended TimeZone (2 zones)     |  10166/  193 |  9718/  183 |
| Extended ZoneManager (1 zone)   |  11314/  213 | 10866/  203 |
| Extended ZoneManager (all)      |  36242/  707 | 35794/  697 |
|---------------------------------+--------------+-------------|
| SystemClock                     |   5662/  282 |  5214/  272 |
| SystemClock+Basic TimeZone      |   9990/  334 |  9542/  324 |
| SystemClock+Extended TimeZone   |  12996/  334 | 12548/  324 |
+--------------------------------------------------------------+

```

## Sparkfun Pro Micro

* Arduino IDE 1.8.13
* SparkFun AVR Boards 1.1.13

```
+--------------------------------------------------------------+
| Functionality                   |    flash/ram |       Delta |
|---------------------------------+--------------+-------------|
| Baseline                        |   3464/  150 |     0/    0 |
|---------------------------------+--------------+-------------|
| LocalDateTime                   |   4772/  263 |  1308/  113 |
| ZonedDateTime                   |   5564/  263 |  2100/  113 |
| Basic TimeZone (1 zone)         |  10012/  327 |  6548/  177 |
| Basic TimeZone (2 zones)        |  10588/  331 |  7124/  181 |
| Basic ZoneManager (1 zone)      |  11476/  353 |  8012/  203 |
| Basic ZoneManager (all)         |  26390/  739 | 22926/  589 |
| Extended TimeZone (1 zone)      |  12792/  327 |  9328/  177 |
| Extended TimeZone (2 zones)     |  13266/  331 |  9802/  181 |
| Extended ZoneManager (1 zone)   |  14416/  353 | 10952/  203 |
| Extended ZoneManager (all)      |  39344/  847 | 35880/  697 |
|---------------------------------+--------------+-------------|
| SystemClock                     |   8646/  422 |  5182/  272 |
| SystemClock+Basic TimeZone      |  12974/  474 |  9510/  324 |
| SystemClock+Extended TimeZone   |  15980/  474 | 12516/  324 |
+--------------------------------------------------------------+

```

## SAMD21 M0 Mini

* Arduino IDE 1.8.13
* Arduino SAMD Boards 1.8.9

```
+--------------------------------------------------------------+
| Functionality                   |    flash/ram |       Delta |
|---------------------------------+--------------+-------------|
| Baseline                        |  11096/ 2368 |     0/    0 |
|---------------------------------+--------------+-------------|
| LocalDateTime                   |  11976/ 2948 |   880/  580 |
| ZonedDateTime                   |  12132/ 2948 |  1036/  580 |
| Basic TimeZone (1 zone)         |  16144/ 2948 |  5048/  580 |
| Basic TimeZone (2 zones)        |  16532/ 2948 |  5436/  580 |
| Basic ZoneManager (1 zone)      |  16732/ 2948 |  5636/  580 |
| Basic ZoneManager (all)         |  35760/ 2948 | 24664/  580 |
| Extended TimeZone (1 zone)      |  18028/ 2948 |  6932/  580 |
| Extended TimeZone (2 zones)     |  18428/ 2948 |  7332/  580 |
| Extended ZoneManager (1 zone)   |  18616/ 2948 |  7520/  580 |
| Extended ZoneManager (all)      |  50840/ 2948 | 39744/  580 |
|---------------------------------+--------------+-------------|
| SystemClock                     |  14300/ 2948 |  3204/  580 |
| SystemClock+Basic TimeZone      |  17716/ 2948 |  6620/  580 |
| SystemClock+Extended TimeZone   |  19664/ 2948 |  8568/  580 |
+--------------------------------------------------------------+

```

(SAMD compiler does not produce RAM usage numbers.)

## ESP8266

* Arduino IDE 1.8.13
* ESP8266 Boards 2.7.4

```
+--------------------------------------------------------------+
| Functionality                   |    flash/ram |       Delta |
|---------------------------------+--------------+-------------|
| Baseline                        | 256700/26776 |     0/    0 |
|---------------------------------+--------------+-------------|
| LocalDateTime                   | 258820/27264 |  2120/  488 |
| ZonedDateTime                   | 259508/27264 |  2808/  488 |
| Basic TimeZone (1 zone)         | 264800/27692 |  8100/  916 |
| Basic TimeZone (2 zones)        | 265040/27692 |  8340/  916 |
| Basic ZoneManager (1 zone)      | 265776/27692 |  9076/  916 |
| Basic ZoneManager (all)         | 285216/27692 | 28516/  916 |
| Extended TimeZone (1 zone)      | 266824/27788 | 10124/ 1012 |
| Extended TimeZone (2 zones)     | 267128/27788 | 10428/ 1012 |
| Extended ZoneManager (1 zone)   | 267800/27788 | 11100/ 1012 |
| Extended ZoneManager (all)      | 300620/27792 | 43920/ 1016 |
|---------------------------------+--------------+-------------|
| SystemClock                     | 262284/27276 |  5584/  500 |
| SystemClock+Basic TimeZone      | 267212/27692 | 10512/  916 |
| SystemClock+Extended TimeZone   | 269476/27788 | 12776/ 1012 |
+--------------------------------------------------------------+

```

## ESP32

* Arduino IDE 1.8.13
* ESP32 Boards 1.0.4

```
+--------------------------------------------------------------+
| Functionality                   |    flash/ram |       Delta |
|---------------------------------+--------------+-------------|
| Baseline                        | 206435/14564 |     0/    0 |
|---------------------------------+--------------+-------------|
| LocalDateTime                   | 218129/16084 | 11694/ 1520 |
| ZonedDateTime                   | 218933/16084 | 12498/ 1520 |
| Basic TimeZone (1 zone)         | 222633/16084 | 16198/ 1520 |
| Basic TimeZone (2 zones)        | 222885/16084 | 16450/ 1520 |
| Basic ZoneManager (1 zone)      | 223461/16084 | 17026/ 1520 |
| Basic ZoneManager (all)         | 242857/16084 | 36422/ 1520 |
| Extended TimeZone (1 zone)      | 224521/16084 | 18086/ 1520 |
| Extended TimeZone (2 zones)     | 224797/16084 | 18362/ 1520 |
| Extended ZoneManager (1 zone)   | 225325/16084 | 18890/ 1520 |
| Extended ZoneManager (all)      | 258133/16084 | 51698/ 1520 |
|---------------------------------+--------------+-------------|
| SystemClock                     | 225717/16188 | 19282/ 1624 |
| SystemClock+Basic TimeZone      | 229317/16188 | 22882/ 1624 |
| SystemClock+Extended TimeZone   | 231269/16188 | 24834/ 1624 |
+--------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* Arduino IDE 1.8.13
* Teensyduino 1.53

```
+--------------------------------------------------------------+
| Functionality                   |    flash/ram |       Delta |
|---------------------------------+--------------+-------------|
| Baseline                        |   7656/ 3048 |     0/    0 |
|---------------------------------+--------------+-------------|
| LocalDateTime                   |  13300/ 4812 |  5644/ 1764 |
| ZonedDateTime                   |  13300/ 4812 |  5644/ 1764 |
| Basic TimeZone (1 zone)         |  22024/ 4812 | 14368/ 1764 |
| Basic TimeZone (2 zones)        |  22760/ 4812 | 15104/ 1764 |
| Basic ZoneManager (1 zone)      |  23680/ 4812 | 16024/ 1764 |
| Basic ZoneManager (all)         |  43240/ 4812 | 35584/ 1764 |
| Extended TimeZone (1 zone)      |  24280/ 4812 | 16624/ 1764 |
| Extended TimeZone (2 zones)     |  24952/ 4812 | 17296/ 1764 |
| Extended ZoneManager (1 zone)   |  25872/ 4812 | 18216/ 1764 |
| Extended ZoneManager (all)      |  58840/ 4812 | 51184/ 1764 |
|---------------------------------+--------------+-------------|
| SystemClock                     |  16224/ 4812 |  8568/ 1764 |
| SystemClock+Basic TimeZone      |  25132/ 4812 | 17476/ 1764 |
| SystemClock+Extended TimeZone   |  27324/ 4812 | 19668/ 1764 |
+--------------------------------------------------------------+

```

